/*
 * @(#)DependencyGroup.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill;
import org.apache.log4j.Category;
import java.util.*;
import java.io.*;
import com.urbancode.lib.registry.*;
import com.urbancode.anthill.util.*;
/**
<<<<<<< DependencyGroup.java
 *
=======
 * @version $Revision: 1.7 $
>>>>>>> 1.5.2.1
 * @author
 */
public class DependencyGroup implements Buildable {
    
    //**************************************************************************
    // CLASS
    //**************************************************************************
    
    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(DependencyGroup.class.getName());
    
    // suffix used by all project files
    static public final String DEPENDENCY_GROUP_FILE_SUFFIX = ".dependencies";

    static public final String ARTIFACT_DIR_NAME        = "artifacts";

    static public final String SCHEDULE_KEY             = "schedule";
    static public final String MEMBER_KEY               = "member";
    static public final String PROJECT_KEY              = "project";
    static public final String DEPENDENCY_KEY           = "dependency";
    static public final String TARGET_KEY               = "target";

    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    protected Anthill anthill = null;
    protected String name = null;
    protected List memberList = new ArrayList();
    protected Map name2member = new HashMap();
    protected File file = null;
    protected RegistryEntry rootEntry = null;
    protected Registry registry = null;
    protected AnthillSchedule schedule = null;
    protected boolean initialized = true;
    
    //--------------------------------------------------------------------------
    public DependencyGroup(Anthill anthill, File regFile) 
    throws Exception {
        this.anthill = anthill;
        setName(FileUtils.getNameMinusExtension(regFile));
        this.file = regFile;
        if (!file.exists()) {
            file.createNewFile();
        }
        // load the registry
        this.registry = Registry.loadFile(file);
        this.rootEntry = registry.getRootRegistryEntry();
        restore();
    }
    
    //--------------------------------------------------------------------------
    protected void restore() 
    throws Exception {
        // set the schedule
        String scheduleName = rootEntry.getKeyValue(SCHEDULE_KEY);
        if (scheduleName != null) {
            schedule = anthill.getSchedule(scheduleName);
            if (schedule != null) {
                schedule.add(this);
            } else {
                log.error("DependencyGroup points to a " +
                          "non existing schedule: " + scheduleName);
                throw new IllegalStateException("DependencyGroup points " +
                                              "to a non existing schedule: " + 
                                              scheduleName);
            }
        }
        
        // now load the members
        Iterator itr = null;
        RegistryEntry memberEntry = null;
        String projectName = null;
        DependencyGroup.Member member = null;
        AnthillProject project = null;
        
        // do the first pass
        itr = rootEntry.getChildRegistryEntries(MEMBER_KEY);
        while (itr.hasNext()) {
            memberEntry = (RegistryEntry) itr.next();
            projectName = memberEntry.getKeyValue(PROJECT_KEY);
            project = anthill.getProject(projectName);
            if (project == null) {
                log.error("DependencyGroup points to a nonexisting " + 
                          "project: " + projectName);
                throw new IllegalStateException ("DependencyGroup points to a " + 
                          "nonexisting project: " + projectName);
            }
            member = createNewMember(project);
        }

        // do the second pass
        Iterator depItr = null;
        String depName = null;
        DependencyGroup.Member depMember = null;
        RegistryEntry depEntry = null;
        String targetName = null;
        ProjectTarget target = null;
        itr = rootEntry.getChildRegistryEntries(MEMBER_KEY);
        while (itr.hasNext()) {
            memberEntry = (RegistryEntry) itr.next();
            projectName = memberEntry.getKeyValue(PROJECT_KEY);
            member = (DependencyGroup.Member) name2member.get(projectName);
            
            // get the iterator of dependencies for the member
            depItr = memberEntry.getKeyValues(DEPENDENCY_KEY);
            while (depItr.hasNext()) {
                depName = (String) depItr.next();
                depMember = (DependencyGroup.Member) name2member.get(depName);
                if (depMember == null) {
                    log.error("DependencyGroup member: " + projectName + 
                              "points to a nonexisting " +
                              "dependency: " + depName);
                    throw new IllegalStateException ("DependencyGroup " + 
                              "member: "+ projectName + 
                              "points to a nonexisting " +
                              "dependency: " + depName);
                }
                depEntry = memberEntry.getChildRegistryEntry(DEPENDENCY_KEY + "." + depName);
/*
                if (depEntry != null) {
                    targetName = depEntry.getKeyValue(TARGET_KEY);
                    if (targetName != null) {
                        target = new ProjectTarget(depMember.getProject(), targetName);
                        depMember.setProjectTarget(target);
                    }
                }
 */
                member.addDependency(depMember);
            }
        }
        
        initialized = true;
    }    
    
    //--------------------------------------------------------------------------
    protected void store() 
    throws Exception {
        // clear the registry
        registry.clear();
        
        // set the schedule
        if (schedule != null) {
            rootEntry.setKeyValue(SCHEDULE_KEY, schedule.getScheduleName());
        }
        
        // now store the members
        Iterator itr = null;
        Iterator depItr = null;
        RegistryEntry regEntry = null;
        RegistryEntry memberEntry = null;
        DependencyGroup.Member member = null;
        DependencyGroup.Member depMember = null;
        ProjectTarget target = null;
        
//        regEntry = rootEntry.createChildRegistryEntry(MEMBER_KEY);
        
        itr = getMemberIterator();
        while (itr.hasNext()) {
            member = (DependencyGroup.Member) itr.next();
//            memberEntry = rootEntry.createChildRegistryEntry(member.getName());
//            memberEntry.addKeyValue(PROJECT_KEY, member.getName());
            memberEntry = rootEntry.addKeyValue(MEMBER_KEY + "." + PROJECT_KEY, member.getName());
            
            depItr = member.getDependencyIterator();
            while (depItr.hasNext()) {
                depMember = (DependencyGroup.Member) depItr.next();
                memberEntry.addKeyValue(DEPENDENCY_KEY, depMember.getName());
/*
                target = depMember.getProjectTarget();
                if (target != null) {
                    memberEntry.addKeyValue(TARGET_KEY, target.getName());
                }
 */
            }
        }
        
    }
    
    //--------------------------------------------------------------------------
    /**
     * If the Group already exists, then we need to make sure that it was 
     * not renamed.  If it was renamed, then we need to delete it's old 
     * file and save it to a new file (a file with the correct name).
     * If the Group is newly created, then we need to save it to a new file.
     */
    public void save() 
    throws Exception {
        
        // create the registry
        store();
        
        // now save the registry to file
        if (file != null) {
            if (!name.equals(FileUtils.getNameMinusExtension(file))) {
                // the file exists but the Group name has been changed
                // we need to delete it so that we can save the Group into 
                // a file whose name matches the name of the Group
                file.delete();
                file = null;
            } else {
                // file exists and is the same name as the Group, 
                // save into the file
                try {
                    registry.updateRegistryFile();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                    throw e;
                }
            }
        } 
        
        // we could have deleted the file above if the name of the Group 
        // changed
        if (file == null) {
            // we need to create the file
            File projectsDir = anthill.getAnthillProjectsDir();
            file = new File(projectsDir, name + DEPENDENCY_GROUP_FILE_SUFFIX);

            // try saving the registry into the file
            try {
                registry.setRegistryFile(file);
                registry.updateRegistryFile();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
                throw e;
            }
        }
        log.debug("Saved");
    }
    
    //--------------------------------------------------------------------------
    public void doScheduleNameChanged(AnthillSchedule mySchedule) 
    throws Exception {
        save();
    }
    
    //--------------------------------------------------------------------------
    public String getName() {
        return name;
    }
    
    //--------------------------------------------------------------------------
    public void setName(String name) {
        String oldName = this.name;
        this.name = name;
        if (oldName != null && !oldName.equals(name)) {
            anthill.doDependencyGroupNameChange(oldName);
/*
            for (Iterator memberItr = memberList.iterator(); memberItr.hasNext();) {
                ((DependencyGroup.Member)memberItr.next())
                    .initializeBuildDefinition(this);
            }
 */
        }
    }
    
    //--------------------------------------------------------------------------
    public void setSchedule(AnthillSchedule newSchedule) {
        if (schedule != null && !schedule.equals(newSchedule)) {
            schedule.remove(this);
        }
        if (newSchedule != null && !newSchedule.equals(schedule)) {
            newSchedule.add(this);
        }
        schedule = newSchedule;
    }
    
    //--------------------------------------------------------------------------
    public AnthillSchedule getSchedule() {
        return schedule;
    }
    
    //--------------------------------------------------------------------------
    public DependencyGroup.Member createNewMember(AnthillProject project) {
        DependencyGroup.Member member = null;
        member = new DependencyGroup.Member(project);//, this);
        memberList.add(member);
        name2member.put(project.getProjectName(), member);
        return member;
    }

    //--------------------------------------------------------------------------
    public void deleteMember(DependencyGroup.Member member) {
        name2member.remove(member.getName());
        memberList.remove(member);
        // remove from Registry
    }
    
    //--------------------------------------------------------------------------
    public DependencyGroup.Member getMember(String name) {
        return (DependencyGroup.Member) name2member.get(name);
    }
    
    //--------------------------------------------------------------------------
    public Iterator getMemberIterator() {
        return memberList.iterator();
    }
    
    //--------------------------------------------------------------------------
    public Iterator getPotentialProjectIterator() {
        List tempList = new ArrayList();
        Iterator projectItr = anthill.getProjectIterator();
        while (projectItr.hasNext()) {
            tempList.add(projectItr.next());
        }
        
        DependencyGroup.Member member = null;
        Iterator itr = getMemberIterator();
        while (itr.hasNext()) {
            member = (DependencyGroup.Member) itr.next();
            tempList.remove(member.getProject());
        }
        
        return tempList.iterator();
    }
    
    //--------------------------------------------------------------------------
    public void run() {
        List buildList = new ArrayList(memberList);
        Iterator memberItr = null;
        DependencyGroup.Member member = null;
        log.debug("run  - inside DepGroup " + this.getName());
        DependencyGroup.Member tempMember = null;
        DependencyGroup.Scope scope = new DependencyGroup.Scope();
        int i = 0;
        
        while (!buildList.isEmpty()) {
            i = 0;
            while (i < buildList.size()) {
                member = (DependencyGroup.Member) buildList.get(i);
                if (member.areDependenciesBuilt(scope)) {
                    log.debug("inside run " +member.getName());
                    log.debug("inside run " +member.getBuildDefinition().getForceBuildFlag());
                    member.build(scope);
                    buildList.remove(i);                    
                } else {
                    i++;
                }
            }
        }
    }
    
    //**************************************************************************
    // Member INNER CLASS
    //**************************************************************************
    public class Member {
        
        //**********************************************************************
        // INSTANCE
        //**********************************************************************
        protected AnthillProject project = null;
        protected List dependencyList = new ArrayList();
        protected Map name2dependency = new HashMap();
        
        //----------------------------------------------------------------------
        protected Member(AnthillProject project) {//, DependencyGroup group) {
            this.project = project;
        }

        //----------------------------------------------------------------------
        public BuildDefinition getBuildDefinition() {
            ProjectProperties properties = project.getProperties();

            BuildDefinition buildDef = new BuildDefinition();
           
            buildDef.setProject(project);
            ProjectTarget target = new ProjectTarget(
                project, 
                properties.getDependencyTargetName());
            // set the publish dir for the dependency group
            File publishRootDir = properties.getAnthillDefaultPublishDir();            
            File properPath = FileUtils.makeProperPath(
                publishRootDir, 
                "depDir" + File.separatorChar + DependencyGroup.this.getName());
            if (!properPath.exists()) {
                properPath.mkdirs();
            }
            String path = properPath.getAbsolutePath();
            // add the build parameter whose value is the publish dir
            target.addBuildSysParam(
                properties.getDependencyArtifactProperty(), 
                path);
            buildDef.setTarget(target);
            
            return buildDef;
        }
        
        //----------------------------------------------------------------------
        public String getName() {
            String name = null;
            if (project != null) {
                name = project.getProjectName();
            } else {
                name = null;
            }
            return name;
        }
        
        //----------------------------------------------------------------------
        public AnthillProject getProject() {
            return project;
        }
        
        //----------------------------------------------------------------------
        public void setProject(AnthillProject project) {
            this.project = project;
        }
       
        //----------------------------------------------------------------------
        public ProjectTarget getProjectTarget() {
            BuildDefinition buildDef = getBuildDefinition();
            return buildDef.getTarget();
        }
        
        //----------------------------------------------------------------------
        public void setProjectTarget(ProjectTarget target) {
            BuildDefinition buildDef = getBuildDefinition();
            buildDef.setTarget(target);
        }
        
        //----------------------------------------------------------------------
        public void addDependency(DependencyGroup.Member dependency) {
            dependencyList.add(dependency);
            name2dependency.put(dependency.getName(), dependency);
        }
        
        //----------------------------------------------------------------------
        public void deleteDependency(DependencyGroup.Member dependency) {
            dependencyList.remove(dependency);
            name2dependency.remove(dependency.getName());
        }
        
        //----------------------------------------------------------------------
        public DependencyGroup.Member getDependency(String name) {
            return (DependencyGroup.Member) name2dependency.get(name);
        }
        
        //----------------------------------------------------------------------
        public Iterator getDependencyIterator() {
            return dependencyList.iterator();
        }
        
        //----------------------------------------------------------------------
        public Iterator getPotentialDependencyIterator() {
            List tempList = (List) ((ArrayList)memberList).clone();
            tempList.removeAll(dependencyList);
            tempList.remove(this);
            
            // Now we check whether this is a dependency of 
            // any of the entries in tempList
            Iterator itr = tempList.iterator();
            DependencyGroup.Member member = null;
            while (itr.hasNext()) {
                member = (DependencyGroup.Member) itr.next();
                if (member.isDependency(this)) {
                    itr.remove();
                }
            }
            
            return tempList.iterator();
        }
        
        //----------------------------------------------------------------------
        public boolean isDependency(DependencyGroup.Member dependency) {
            DependencyGroup.Member tempDep = null;
            boolean result = false;
            
            for (int i = 0; i < dependencyList.size(); i++) {
                tempDep = (DependencyGroup.Member) dependencyList.get(i);
                if (tempDep.equals(dependency)) {
                    result = true;
                    break;
                }
                if (tempDep.isDependency(dependency)) {
                    result = true;
                    break;
                }
            }
            return result;
        }

        //----------------------------------------------------------------------
        public boolean isBuilt(DependencyGroup.Scope scope) {
            return scope.isBuilt(this);
        }
        
        //----------------------------------------------------------------------
        public void build(DependencyGroup.Scope scope) {
            BuildDefinition buildDef = getBuildDefinition();
            buildDef.run();
            scope.setBuilt(this, true);
        }
        
        //----------------------------------------------------------------------
        public boolean areDependenciesBuilt(DependencyGroup.Scope scope) {
            DependencyGroup.Member tempDep = null;
            boolean result = false;
            
            if (dependencyList.isEmpty()) {
                result = true;
            } else {
                result = true;
                for (int i = 0; i < dependencyList.size(); i++) {
                    tempDep = (DependencyGroup.Member) dependencyList.get(i);
                    if (!tempDep.isBuilt(scope)) {
                        result = false;
                        break;
                    }
                }
            }
            return result;
        }

//Merge added this        
        //----------------------------------------------------------------------
        protected void initializeBuildDefinition(DependencyGroup group) {
            ProjectProperties properties = project.getProperties();
            BuildDefinition buildDef = null;
            buildDef = new BuildDefinition();
            buildDef.setProject(project);
            ProjectTarget target = new ProjectTarget(
                project, 
                properties.getDependencyTargetName());
            // set the publish dir for the dependency group
            File publishRootDir = properties.getAnthillDefaultPublishDir();            
            File properPath = FileUtils.makeProperPath(
                publishRootDir, 
                "depDir" + File.separatorChar + group.getName());
            if (!properPath.exists()) {
                properPath.mkdirs();
            }
            String path = properPath.getAbsolutePath();
//            target.setPublishDir(path);
            // add the build parameter whose value is the publish dir
            
            target.addBuildSysParam(
                properties.getDependencyArtifactProperty(), 
                path);
       
            buildDef.setTarget(target);
        }

    }

    //**************************************************************************
    // Scope INNER CLASS
    //**************************************************************************
    public static class Scope {
        protected Set builtSet = new HashSet();
        
        //----------------------------------------------------------------------
        public void setBuilt(DependencyGroup.Member dependency, boolean flag) {
            if (flag) {
                builtSet.add(dependency);
            } else {
                builtSet.remove(dependency);
            }
        }
            
        //----------------------------------------------------------------------
        public boolean isBuilt(DependencyGroup.Member dependency) {
            boolean result = builtSet.contains(dependency);
            return result;
        }
    }
}