/*
 * @(#)ProjectTarget.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill;
import com.urbancode.lib.registry.Registry;
import com.urbancode.lib.registry.RegistryEntry;
import org.apache.log4j.Category;
import java.util.*;
/**
 * @version $Revision: 1.4 $
 * @author Maciej Zawadzki
 * @author Jason Buck
 */
public class ProjectTarget {
    
    //**************************************************************************
    // CLASS
    //**************************************************************************
    
    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(ProjectTarget.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    protected AnthillProject project = null;
    protected String name = null;
    protected Registry registry = null;
    protected RegistryEntry targetRegEntry = null;
    
    //--------------------------------------------------------------------------
    public ProjectTarget(AnthillProject project, String name) {
        this.project = project;
        this.name = name;
        this.registry = Registry.createRegistry();
        this.targetRegEntry = registry.createChildRegistryEntry(
            ProjectProperties.REGISTRY_ENTRY_KEY);        
    }
    
    //--------------------------------------------------------------------------
    public AnthillProject getProject() {
        return project;
    }
    
    //--------------------------------------------------------------------------
    public String getName() {
        return name;
    }
    
    //--------------------------------------------------------------------------
    public String getPublishDir() {
        return targetRegEntry.getKeyValue(
            ProjectProperties.PUBLISH_DIR_NAME);
    }
    
    //--------------------------------------------------------------------------
    public void setPublishDir(String publishDir) {
        targetRegEntry.setKeyValue(
            ProjectProperties.PUBLISH_DIR_NAME, publishDir);
    }
    
    //--------------------------------------------------------------------------
    // this method doesnt' really add a sys param anymore, since we don't
    // have those, it really just goes into the addBuildAntParam
    public void addBuildSysParam(String name, String value) {

        RegistryEntry regEntry = targetRegEntry.addKeyValue(
            ProjectProperties.BUILD_ANT_PARAM_NAME_KEY, "-D"+name);
        regEntry.addKeyValue(
            ProjectProperties.BUILD_PUB_PARAM_VALUE_KEY, "-D"+name+"="+value);

    }
    
    //--------------------------------------------------------------------------
//    public Iterator getBuildSysParamRegEntryIterator() {
//        return targetRegEntry.getChildRegistryEntries(
//           ProjectProperties.BUILD_SYS_PARAM_NAME_KEY);
//    }
    
    //--------------------------------------------------------------------------
    protected RegistryEntry getRegistryEntry() {
        return targetRegEntry;
    }
    
}
