/*
 * PropertyFilter.java
 *
 * Created on July 10, 2002, 4:38 PM
 */

package com.urbancode.anthill;
import com.urbancode.lib.registry.*;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.File;
import java.util.Map;
import java.util.Iterator;
import org.apache.log4j.Category;

/**
 *
 * @author  spb
 */
public class PropertyFilter {
    
    static public final String REGISTRY_ENTRY_KEY        = "anthill";
    static public final String BUILD_SCRIPT_SYS_PARAMS   = "build";
    static public final String BUILD_SCRIPT_ANT_PARAMS   = "build.ant";
    static public final String PUBLISH_SCRIPT_SYS_PARAMS = "publish";
    static public final String PUBLISH_SCRIPT_ANT_PARAMS = "publish.ant";
    static public final String PARAMS                    = "params";
    static public final String VALUE                     = "value";
    static public final String BUILD_ANT_PARAM_NAME_KEY  = "build.ant.params.name";
    static public final String PUB_ANT_PARAM_NAME_KEY    = "publish.ant.params.name";
    static public final String BUILD_ANT_PARAM_VALUE_KEY = "build.ant.params.value";
    static public final String PUB_ANT_PARAM_VALUE_KEY   = "publish.ant.params.value";
    static public final String REGISTRY_FORMAT_VERSION  = "registry.format.version";
    protected RegistryEntry projectRegEntry;
    
    
    static private Category log = 
        Category.getInstance(PropertyFilter.class.getName());
    
    /** Creates a new instance of PropertyFilter */
    public PropertyFilter() {
        
    }
    
    protected void backupFile(File file, String version) {
        log.debug("Found old project file: " + file.getAbsolutePath());
        File backupFile = new File(file.getAbsolutePath()+"_ver"+version+".old");
        log.debug("Going to create backup file: " + backupFile.getAbsolutePath());
        
         if (backupFile.exists()) { // this shouldn't happen
             log.debug("Backup file already exists, deleting old version");
             if (!backupFile.delete()) {
                 // here we couldn't delete the file...
                 log.error("Error deleting previous project configuration file");
             }
            // copy the file
         }
         try {
             backupFile.createNewFile();
             FileReader in = new FileReader(file);
             FileWriter out = new FileWriter(backupFile);
             int c;

             while ((c = in.read()) != -1)
                 out.write(c);

             in.close();
             out.close();
         }
         catch (Exception e) {
             log.error("Error backing up registry file: " + e.getMessage());
         }            
    }
    
    // returns -1 if version<expected, 0 if equal, 1 if version>expected
    protected int isUpToDate(String version, String expectedVersion) {
        // so this all depends on how many characters each of the two
        // expected versions has.  version is what's in the properties file
        // expected is what the filter that's running is looking for
        if (version == null) {
            return -1;
        }
        if (expectedVersion == null) {
            log.error("You must supply a non-null expectedVersion!");
            return 1;
        }
        
        if (version.equals(expectedVersion)) {
			System.out.println("versions identical, no update necessary.");
			return 0;
	}
	// 2 main cases left, same number of dots is covered by for loop logic
	// ie it will exit before the for loop finishes
	int verDig = 0;
	int expDig = 0;
	boolean done = false;
	boolean verMore = true;
	int forUpTo = 0;
	int verDots = getNumDots(version);
	int expDots = getNumDots(expectedVersion);
	
	// look at the number of digits we have to deal with by seeing how 
	// many dots there are only loop to lower number
	forUpTo = java.lang.Math.min(verDots, expDots);
	verMore = (verDots > forUpTo);
	
	for (int i = 0; i <= forUpTo; i++) {
		verDig = getInt(version, i);
		expDig = getInt(expectedVersion, i);
		if (verDig > expDig) {
			return 1;
		}
		else if (verDig < expDig) {
			return -1;
		}
		// here you have to keep going
	}
	// here you're done, so they are equal up to the lowest number of digits...
	if (verMore) {
            return 1;
	}
	else {
            return -1;
	}
    }
    
    public int getNumDots(String verStr) {
        int count = 0;
	for (int i = 0; i < verStr.length(); i++) {
            if (verStr.charAt(i) == '.') count++;
	}
	return count;
    }
	
    public int getInt(String verStr, int digitNum) {
    // so here we are given which digit to go for, then grab it
    // ie 0.1.2.3.4.5...notice we start at 0 not 1...
        try {
            if (digitNum == 0) {
                return Integer.parseInt(verStr.substring(0,1));
            }
            int lastIndex = 0;
            int dotIndex = 0;
            // else digitNum is >= 1
            for (int i = 0; i < digitNum; i++) {
                // so for each one they want, you find that many dots, and 
                // take the next number
                dotIndex = verStr.indexOf(".", lastIndex);
                if (dotIndex == -1) return -1;
                    lastIndex = dotIndex+1; // move the index up so we don't keep getting the same one
                }
                // so now we have the right one, so you need to grab the next digit
                // if there is another dot, use the index of again, it could also be the end of the
                // string, if there are no more dots, so you want from dot Index to the end of the string
                int moreDotIndex = verStr.indexOf(".", lastIndex);
                if (moreDotIndex < 0) {
                    return (Integer.parseInt(verStr.substring(dotIndex+1, verStr.length())));
                }
                else {
                    return (Integer.parseInt(verStr.substring(dotIndex+1, moreDotIndex)));
                }
        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
            System.exit(-1);
        }
        return -1;
    }   
}
