/*
 * @(#)ProjectPropertyFileVersionAdapter.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.adapter;

import org.apache.log4j.Category;
import com.urbancode.anthill.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
/**
 *
 * @version $Revision: 1.2 $
 */
public class ProjectPropertyFileVersionAdapter extends VersionAdapter {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(ProjectPropertyFileVersionAdapter.class.getName());
    
    // property key for the version property in the project registry
    static public final String VERSION_NUMBER_PROP_KEY = "version";

    static private final int PREFIX = 1;
    static private final int DIGIT = 2;
    static private final int POSTFIX = 3;

    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    protected String versionFile = null;
    protected ProjectProperties properties = null;

    /**
     * Create a new <code>UrbanCodeVersionAdapter</code>.
     */
    public ProjectPropertyFileVersionAdapter() {
    }
    
    protected void setAnthillProject(AnthillProject project) {
        this.project = project;
        this.properties = project.getProperties();
    }
    
    /**
     * Increment the project with the version increment. Immediately updates
     * the version file with the new version and returns the new version.
     *
     * @return  string representation of the new version of the project
     */
    public String incrementVersion() 
    throws Exception {
        String branch = project.getRepositoryAdapter().getBranchName();
        
        // get the Version 
        String versionStr = properties.getProperty(VERSION_NUMBER_PROP_KEY);
        String newVersionStr = null;
        log.info("Incrementing version: "+versionStr);
        
        if (branch != null) {
            if (versionStr.indexOf(branch) == -1) {
                throw new IllegalStateException("Version string is illegal.  " +
                            "It does not contain the Branch name. " +
                            "versionStr: " + versionStr);
            }
        }
        
        newVersionStr = incrementVersion0(versionStr);
        
        // set the new version
        properties.setProperty(VERSION_NUMBER_PROP_KEY, newVersionStr);

        //  return the new version property
        return (newVersionStr);
    }
    
    public String getCurrentVersion()
    throws Exception {        
        String versionStr = properties.getProperty(VERSION_NUMBER_PROP_KEY);
        return versionStr;
    }

    public void setCurrentVersion(String newVersion)
    throws Exception {        
        log.info("setting version: "+newVersion);
        
        // set the new version
        properties.setProperty(VERSION_NUMBER_PROP_KEY, newVersion);
    }
    
    protected String incrementVersion0(String versionStr) {
        // incrementing only the build number
        char c;
        StringBuffer prefix = new StringBuffer();
        StringBuffer digits = new StringBuffer();
        StringBuffer postfix = new StringBuffer();
        int state = POSTFIX;

        for (int i = versionStr.length() - 1; i >= 0; i--) {
            c = versionStr.charAt(i);
            if (!Character.isDigit(c)) {
                if (state == POSTFIX) {
                    postfix.insert(0, c);
                } else {
                    if (state == DIGIT) {
                        state = PREFIX;
                    }
                    prefix.insert(0, c);
                }
            } else {
                if (state == POSTFIX || state == DIGIT) {
                    state = DIGIT;
                    digits.insert(0, c);
                } else {
                    prefix.insert(0, c);
                }
            }
        }

        int build = 0;
        try {
            build = Integer.parseInt(digits.toString());
        } catch (NumberFormatException nfe) {
            throw new IllegalStateException("Version String illegal. " + 
                        "Can't parse build number. " +
                        "versionStr: "+versionStr);
        }
        build++;

        String newVersionStr = prefix.toString() + String.valueOf(build) + 
                               postfix.toString();
        
        return newVersionStr;
    }

}