/*
 * @(#)RepositoryAdapterFactory.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */

package com.urbancode.anthill.adapter;
import com.urbancode.anthill.AnthillProject;
import org.apache.log4j.Category;
/**
 * <p>
 * This is a factory to create <code>RepositoryAdapter</code> classes.</p>
 *
 * @author  Ryan Smith
 * @version $Revision 1.0 $
 */
public abstract class RepositoryAdapterFactory {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(RepositoryAdapterFactory.class.getName());

    /**
     * Create the specified <code>RepositoryAdapter</code>.
     *
     * @param repositoryAdapter  the full classname of the adapter including
     *                           the package
     * @param properties  the properties to set on the adapter
     * @return  the requested <code>RepositoryAdapter</code>
     */
    static public RepositoryAdapter getRepositoryAdapter(AnthillProject project,
                                                         String repositoryAdapter)
    throws Exception {

        RepositoryAdapter adapter = null;
        
        try {
            adapter = (RepositoryAdapter) Class.forName(repositoryAdapter).newInstance();
            adapter.setAnthillProject(project);
        } catch (Exception e) {
            log.error("Unable to create repository adapter: " +
                                repositoryAdapter, e);
        }
        
        return adapter;
    }

}
