/*
 * @(#)VersionAdapter.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.adapter;

import org.apache.log4j.Category;
import java.util.Iterator;
import java.io.File;
import com.urbancode.anthill.AnthillProject;
/**
 * <p>
 * An abstract class representing a system of versioning a project with a
 * single version file.</p>
 *
 * @author  Ryan Smith
 */
public abstract class VersionAdapter {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(VersionAdapter.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    protected AnthillProject project = null;

    /**
     * Returns the AnthillProject that this RepositoryAdapter belongs to.
     *
     * @return AnthillProject that this RepositoryAdapter belongs to
     */
    public AnthillProject getAnthillProject() {
        return project;
    }
    
    /**
     * Set the AnthillProject that this RepositoryAdapter belongs to.
     * This method should only be called from the RepositoryAdapterFactory.
     *
     * @param the AnthillProject that this adapter belongs to
     */
    protected void setAnthillProject(AnthillProject project) {
        this.project = project;
    }
    
    /**
     * Increment the project (version file) with the specified increment.
     *
     * @return  the new version of the project after incrmenting
     */
    public abstract String incrementVersion() 
    throws Exception;

    public abstract String getCurrentVersion()
    throws Exception;
    
    public abstract void setCurrentVersion(String newVersion)
    throws Exception;
    
}
    /*
     * Get a <code>Iterator</code> of <code>String</code> objects that
     * represent the possible choices of incrementing the project.
     *
     * @return  the <code>Iterator</code> of increment choices
     *
    public abstract Iterator getIncrementChoices();
*/
