/*
 * @(#)VersionAdapterFactory.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */

package com.urbancode.anthill.adapter;
import com.urbancode.anthill.AnthillProject;
/**
 * <p>
 * This is a factory to create <code>VersionAdapter</code> classes.</p>
 *
 * @author  Ryan Smith
 * @version $Revision 1.0 $
 */
public class VersionAdapterFactory {


    /**
     * Create the specified <code>VersionAdapter</code>.
     *
     * @param versionAdapter  the full classname of the adapter including
     *                        the package
     * @param properties  the properties to set on the adapter
     * @param versionFile  the filename of the version file
     * @return  the requested <code>VersionAdapter</code>
     */
    static public VersionAdapter getVersionAdapter(AnthillProject project,
                                                   String versionAdapter)//,
                                                   //String versionFile )
    throws Exception {

        VersionAdapter adapter = null;
        
        try {
            adapter = (VersionAdapter) Class.forName(versionAdapter).newInstance();
            adapter.setAnthillProject(project);
            //adapter.setVersionFile(versionFile);
        } catch (Exception e) {
            throw new Exception("Unable to create version adapter: " +
                                versionAdapter);
        }
        
        return adapter;
    }
}
