/*
 * @(#)Execute.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.util;
import org.apache.log4j.Category;
/**
 * @version $Revision: 1.3 $
 * @author 
 */
public class Execute {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(Execute.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************

    //--------------------------------------------------------------------------
    static public int executeJavaCommand(String[] commands) 
    throws Exception {
        // log the command if debugging
        if (log.isDebugEnabled()) {
            // put the command into a single string for log output
            StringBuffer cmd = new StringBuffer();
            for (int i=0; i<commands.length; i++) {
                cmd.append(commands[i]).append(" ");
            }
            // print out the command
            log.debug(cmd.toString());
        }

        Process p = null;
        
        // NOTE - No need to attach listeners, they never work for some reason.
        // Ant seems to be getting around this by calling the Main method 
        // directly (check out ExecuteJava in Ant).
        p = Runtime.getRuntime().exec(commands);

        // test for errors when the process is done running
        int exitcode = p.waitFor();

        return exitcode;
    }

    //--------------------------------------------------------------------------
    static public int executeJavaCommand(String command) 
    throws Exception {
        // log the command if debugging
        if (log.isDebugEnabled()) {
            // print out the command
            log.debug(command);
        }

        Process p = null;
        
        // NOTE - No need to attach listeners, they never work for some reason.
        // Ant seems to be getting around this by calling the Main method 
        // directly (check out ExecuteJava in Ant).
        p = Runtime.getRuntime().exec(command);

        // test for errors when the process is done running
        int exitcode = p.waitFor();

        return exitcode;
    }

    /**
     * Run a command by calling a <code>Process</code> and giving it
     * the command line arguments in <code>String[]</code> form.
     *
     * @param commandArray  the array of command-line arguments
     * @param commmand  an arbitrary name to represent the command
     * @param debug  true if the process input and error streams should be 
     *               pumped to Standard out.
     */
    static public int executeCommand(String[] commandArray, 
                                  String commandName, 
                                  boolean debug)
    throws Exception {
        
        if (debug) {
            StringBuffer cmd = new StringBuffer();
            for (int i=0; i<commandArray.length; i++) {
                cmd.append(commandArray[i] + " ");
            }
            log.debug("[" + commandName + "] " + cmd.toString() + "\n");
        }
        
        Process p = null;
        StreamPumper errorPumper = null;
        StreamPumper inputPumper = null;
        int exitcode = 0;
        try {
            p = Runtime.getRuntime().exec(commandArray);

            //Logging the error stream.
            errorPumper = new StreamPumper(p.getErrorStream(), commandName, 
                                 System.out, debug);
            //Logging the input stream.
            inputPumper = new StreamPumper(p.getInputStream(), commandName, 
                                 System.out, debug);
            // start pumping!
            errorPumper.start();
            inputPumper.start();
                
            // test for errors
            exitcode = p.waitFor();
            
        } finally {
            if (errorPumper != null) {
                errorPumper.join();
            }
            if (inputPumper != null) {
                inputPumper.join();
            }
        }
        return exitcode;
    }
    
    /**
     * Run a command by calling a <code>Process</code> and giving it
     * the command line arguments in <code>String[]</code> form.
     *
     * @param command  command-line arguments
     * @param commandName  an arbitrary name to represent the command
     * @param debug  true if the process input and error streams should be 
     *               pumped to Standard out.
     */
    static public int executeCommand(String command, 
                                 String commandName, 
                                 boolean debug)
    throws Exception {
        
        if (debug) {
            log.debug("[" + commandName + "] " + command + "\n");
        }
        
        Process p = null;
        StreamPumper errorPumper = null;
        StreamPumper inputPumper = null;
        int exitcode = 0;
        try {
            p = Runtime.getRuntime().exec(command);

            //Logging the error stream.
            errorPumper = new StreamPumper(p.getErrorStream(), commandName, 
                                 System.out, debug);
            //Logging the input stream.
            inputPumper = new StreamPumper(p.getInputStream(), commandName, 
                                 System.out, debug);
            // start pumping!
            errorPumper.start();
            inputPumper.start();
                
            // test for errors
            exitcode = p.waitFor();
            
        } finally {
            if (errorPumper != null) {
                errorPumper.join();
            }
            if (inputPumper != null) {
                inputPumper.join();
            }
        }
        return exitcode;
    }
    
}
