/*
 * @(#)AnthillAdminServlet.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.web.admin;           
import org.apache.log4j.Category;
import javax.servlet.*;
import javax.servlet.http.*;
import com.urbancode.anthill.*;
import com.urbancode.lib.butler.*;
import java.io.*;
import java.util.*;
/** 
 *
 * @author  MACIEJ
 * @version 
 */
public class AnthillAdminServlet extends HttpServlet {
   
    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(AnthillAdminServlet.class.getName());
    
    // name of initialization parameter
    static private final String CONFIG_DIR_KEY = "config.dir";

    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    /** Initializes the servlet.
    */  
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        ServletContext context = config.getServletContext();
        
        String userHomeName = System.getProperty("user.home");
        File anthillPropFile = new File(userHomeName, ".anthill.properties");
        if (anthillPropFile.exists()) {
            try {
                FileInputStream fin = new FileInputStream(anthillPropFile);
                Properties anthillProps = new Properties();
                anthillProps.load(fin);
                fin.close();

                String anthillRootDirStr = 
                             anthillProps.getProperty(Anthill.ANTHILL_ROOT_DIR_KEY);
                System.setProperty(Anthill.ANTHILL_ROOT_DIR_KEY, anthillRootDirStr);

                // ---------------------------------------------------------------------
                // instantiate Anthill
                // ---------------------------------------------------------------------
                Anthill anthill = Anthill.getAnthill();
                context.setAttribute(WebKeys.AnthillKey, anthill);
                anthill.runDaemon();
            } catch (Throwable t) {
                t.printStackTrace();
            }
        }
    }

    /** Destroys the servlet.
    */  
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, java.io.IOException {
        ServletContext context = getServletContext();
        Anthill anthill = (Anthill)context.getAttribute(WebKeys.AnthillKey);
        if (anthill != null) {
            req.setAttribute(WebKeys.AnthillKey, anthill);
            context.getRequestDispatcher(ScreenNames.mainScreen).forward(req, res);
        } else {
            context.getRequestDispatcher(ScreenNames.configureScreen).forward(req, res);
        }
    } 

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Short description";
    }

}
