/*
 * @(#)AnthillDeleteServlet.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */

package com.urbancode.anthill.web.admin;           
import org.apache.log4j.Category;
import javax.servlet.*;
import javax.servlet.http.*;
import com.urbancode.anthill.*;
import java.io.PrintWriter;
import java.io.File;
import java.util.*;

/** 
 *
 * @author  SPB
 * @author Matthew Gavlak
 * @version 
 */
public class AnthillDeleteStepOneServlet extends HttpServlet {

//**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(AnthillDeleteStepOneServlet.class.getName());
   
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    /** Initializes the servlet.
    */  
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        ServletContext context = config.getServletContext();
        
        // little bit to experiment with here...might not have to do anything
    }
    
    /** Destroys the servlet.
    */  
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, java.io.IOException {
        
        ServletContext context = getServletContext();
        Anthill anthill = (Anthill)context.getAttribute(WebKeys.AnthillKey);
        AnthillProject project = null;
        AnthillSchedule schedule = null;
        Iterator projIterator = null;
        Iterator depGroupIterator = null;
        Buildable projectDep = null;
        DependencyGroup depGroup = null;
        StringBuffer message = new StringBuffer();
        String deleteType = req.getParameter(WebKeys.DeleteTypeKey);
        String deleteName = req.getParameter(WebKeys.DeleteNameKey);
        boolean error = false;
        
        log.debug("processing delete servlet");        
        if ((deleteType == null) || (deleteType.trim().equals("")) ||
            (deleteName == null) || (deleteName.trim().equals(""))) {
                error = true;
        }
        if (!error) {
            try {
                if (deleteType.equalsIgnoreCase("schedule")) {
                    //check if we can delete
                 
                    //check for projects using this schedule
                    schedule = anthill.getSchedule(deleteName);
                    projIterator = schedule.getProjectIterator();
                    while (projIterator.hasNext()) {
                        projectDep = (Buildable)projIterator.next();
                        String name = projectDep.getName();
                            //cant delete - schedule has projects,send message back somehow
                            error = true;
                            message.append("This schedule can't be deleted, it is being used by  ");
                            message.append(name).append("<br>"); 
                            req.setAttribute("errorMessage", message);                            
                    }

                }
                
                else if (deleteType.equalsIgnoreCase("dependencyGroup")) {
                    // dependency group can always be deleted   
                    log.debug("deleting group");        
                }
                
                else if (deleteType.equalsIgnoreCase("project")) {
                    //check to make sure its not envolved with dependency group
                    depGroupIterator = anthill.getDependencyGroupIterator();
                    while (depGroupIterator.hasNext()) {
                        depGroup = (DependencyGroup)depGroupIterator.next();
                        Iterator members = depGroup.getMemberIterator();
                        while (members.hasNext()) {
                            DependencyGroup.Member member = (DependencyGroup.Member)members.next();
                            String memName = member.getName();
                            if (memName.equals(deleteName)) {
                                //error
                                error = true;
                                message.append("This project can't be deleted, it is member of Dependency Group ");
                                message.append(" " + depGroup.getName() + "<br>");
                                req.setAttribute("errorMessage", message);
                                log.debug("Project is member of Dependency Group can't delete");
                            }
                        }
                    }
                    
                } else {
                    // this is a problem
                    log.debug("<BR> DeleteType not defined: " + deleteType);
                    error = true;
                }       
            } catch (Exception e) {
                log.debug(e.getMessage() , e);
                message.append(e.getMessage());
                req.setAttribute("errorMessage", message);
                error = true;
            }
    
            req.setAttribute(WebKeys.AnthillKey, anthill);

        }
        if (error) {
            context.getRequestDispatcher(ScreenNames.errorScreen).forward(req, res);
        }
        else {
            context.getRequestDispatcher(ScreenNames.deleteStepTwoServlet).forward(req, res);
        }
    } 

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Short description";
    }

}