/*
 * @(#)AnthillPropertiesUpdateServlet.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.web.admin;           
import org.apache.log4j.Category;
import javax.servlet.*;
import javax.servlet.http.*;
import com.urbancode.anthill.*;
import java.util.*;
/** 
 *
 * @author  MACIEJ
 * @version 
 */
public class AnthillPropertiesUpdateServlet extends HttpServlet {
   
    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(AnthillPropertiesUpdateServlet.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    /** Initializes the servlet.
    */  
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
    }

    /** Destroys the servlet.
    */  
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, java.io.IOException {
        ServletContext context = getServletContext();
        Anthill anthill = (Anthill)context.getAttribute(WebKeys.AnthillKey);
        AnthillConfig anthillConfig = anthill.getAnthillConfig();
        boolean error = false;
        
        synchronized (anthillConfig) {
            Enumeration propNameEnum = req.getParameterNames();
            String propName = null;
            String propValue = null;
            while (propNameEnum.hasMoreElements()) {
                propName = (String)propNameEnum.nextElement();
                log.debug(propName + "=" + req.getParameter(propName));
                if (propName.equals("ant.home") ||
                    propName.equals("version.adapter") ||
                    propName.equals("repository.adapter") ||
                    propName.equals("repository.cvs.root") ||
//                    propName.equals("work.dir") ||
                    propName.equals("publish.dir.default") ||
                    propName.equals("mail.host") ||
                    propName.equals("mail.from") ||
                    propName.equals("server") ||
//                    propName.equals("lock.version.file") ||
                    propName.equals("dependency.target") ||
                    propName.equals("dependency.artifact.property")) {
                        propValue = req.getParameter(propName);
                        log.debug("property name: "+propName+", value: "+propValue);
                        anthillConfig.setProperty(propName, propValue);
                }
            }
            try {
                anthillConfig.save();
            } catch (Exception e) { 
                log.error(e.getMessage(), e);
                error = true;
            }
        }
        
        if (!error) {
            context.getRequestDispatcher(ScreenNames.AnthillAdminServlet).forward(req , res);
        } else {
            context.getRequestDispatcher(ScreenNames.errorScreen).forward(req , res);
        }
    } 

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Short description";
    }

}
