/*
 * @(#)ConfigureAnthillServlet.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.web.admin;           
import org.apache.log4j.Category;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.*;
import com.urbancode.anthill.*;
import com.urbancode.anthill.util.*;
/** 
 *
 * @author  MACIEJ
 * @version 
 */
public class ConfigureAnthillServlet extends HttpServlet {
   
    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(ConfigureAnthillServlet.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    /** Initializes the servlet.
    */  
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
    }

    /** Destroys the servlet.
    */  
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, java.io.IOException {
        boolean error = false;
        ServletContext context = getServletContext();
        try {
            String anthillRootDirStr = req.getParameter("anthill.home");

            // store the anthill home dir in a property file in the user's
            // home directory
            Properties anthillProps = new Properties();
            anthillProps.setProperty(Anthill.ANTHILL_ROOT_DIR_KEY, anthillRootDirStr);

            String userHomeName = System.getProperty("user.home");
            File anthillPropFile = new File(userHomeName, ".anthill.properties");
            anthillPropFile.createNewFile();
            FileOutputStream fos = new FileOutputStream(anthillPropFile);

            anthillProps.store(fos, "Anthill Home Directory");
            fos.close();

            // ---------------------------------------------------------------------
            // instantiate Anthill
            // ---------------------------------------------------------------------
            System.setProperty(Anthill.ANTHILL_ROOT_DIR_KEY, anthillRootDirStr);
            Anthill anthill = Anthill.getAnthill();
            context.setAttribute(WebKeys.AnthillKey, anthill);
            anthill.runDaemon();
        } catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (!error) {
            context.getRequestDispatcher(ScreenNames.AnthillAdminServlet).forward(req , res);
        } else {
            context.getRequestDispatcher(ScreenNames.errorScreen).forward(req , res);
        }
    } 

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Short description";
    }

}
