/*
 * @(#)ProjectPropertiesUpdateServlet.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.web.admin;           
import com.urbancode.anthill.*;
import com.urbancode.lib.registry.RegistryEntry;
import org.apache.log4j.Category;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
/** 
 *
 * @author  MACIEJ
 * @version 
 */
public class ProjectPropertiesUpdateServlet extends HttpServlet {
   
    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(ProjectPropertiesUpdateServlet.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    /** Initializes the servlet.
    */  
    public void init(ServletConfig config) throws ServletException {
        log.debug("ProjectPropertiesUpdateServlet.init()");
        super.init(config);
    }

    /** Destroys the servlet.
    */  
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, java.io.IOException {
        log.debug("ProjectPropertiesUpdateServlet.processRequest()");
        ServletContext context = getServletContext();
        Anthill anthill = (Anthill)context.getAttribute(WebKeys.AnthillKey);
        AnthillProject project = null;
        String projectName = req.getParameter(WebKeys.ProjectNameKey);
        boolean error = false;

        if (projectName != null) {
            project = anthill.getProject(projectName);
            if (project == null) {
                error = true;
            }
        } else {
            error = true;
        }
        log.debug("project=" + project);
        if (!error) {
            synchronized (project) {
                ProjectProperties properties = project.getProperties();
                RegistryEntry regEntry = null;
                String userName = null;
                String userValue = null;
                userName = req.getParameter("newUserName");
                userValue = req.getParameter("newUserValue");
                Enumeration propNameEnum = req.getParameterNames();
                String propName = null;
                String propValue = null;
                while (propNameEnum.hasMoreElements()) {
                    propName = (String)propNameEnum.nextElement();
                    propValue = req.getParameter(propName);
                    log.debug(propName + "=" + propValue);
                    // do the project name
                    if (propName.equals("project.name")) {
                        project.setProjectName(propValue);
                    }
                                     

                    if (propName.equals("version.adapter") ||
                        propName.equals("version.file") ||
                        propName.equals("version.lock.file") ||
                        propName.equals("repository.adapter") ||
                        propName.startsWith("repository.") ||
                        propName.equals("build.script") ||
                        propName.equals("publish.script") ||
                        propName.equals("publish.dir") ||
                        propName.equals("publish.url")) {
                            properties.setProperty(propName, propValue);
                    }
                    
                    // set schedule name
                    if (propName.equals("schedule")) {
                        properties.setScheduleName(propValue);
                    }
                    
                    //remove email if checkbox deleted, or if it was not filled in, otherwise set key value
                    if (propName.startsWith("users")) {
                        if (propValue.equals(""))  {
                            log.debug("removing empty name: "+propName+", value: "+propValue); 
                            properties.removeNotificationUser(propName);                                                     
                        } else {
                            log.debug("saving name: "+propName+", value: "+propValue);                                 
                            properties.setProperty(propName, propValue);                               
                        }
                    }
                    
                    //checked box for user- remove
                    if (propName.startsWith("remove")) {
                        log.debug("removing userName: "+propValue);
                        properties.removeNotificationUser(propValue);
                    }                    
                    
                    //checked box for build.params - remove
              
                    if (propName.startsWith("deleteBAnt")) {
                        log.debug("removing paramName: "+propValue);
                        properties.removeBuildParamAntProperty(propValue);
                    }                    

                    if (propName.startsWith("deletePAnt")) {
                        log.debug("removing paramName: "+propValue);
                        properties.removePublishParamAntProperty(propValue);
                    }                    
                    
                    //save new build.params
                    
                    if (propName.equals("newBuildAntParamValue")) {
                        if (!propValue.equals("")) {
                            properties.addBuildAntParam(propValue);
                        }
                    }
                    //save new publish.params
                    
                    if (propName.equals("newPublishAntParamValue")) {
                        if (!propValue.equals("")) {
                            properties.addPubAntParam(propValue);
                       }
                    }
                    
                    //save new users
                    if (propName.equals("newUserName")) {
                        if(!userValue.equals("") && (userName != null)) {
                            userName = "users."+userName;
                            log.debug("saving userName: "+propName+", value: "+propValue);
                            properties.setProperty(userName, userValue);
                        }
                    }
                    
                }
                try {
                    project.save();
                } catch (Exception e) { 
                    log.error(e.getMessage(), e);
                    error = true;
                }
            }
        }
         
        if (!error) {
            req.setAttribute(WebKeys.ProjectKey, project);
            context.getRequestDispatcher(ScreenNames.AnthillAdminServlet).forward(req , res);
        } else {
            context.getRequestDispatcher(ScreenNames.errorScreen).forward(req , res);
        }
    } 

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Short description";
    }

}
