/*
 * @(#)RepositoryPropertiesViewServlet.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.web.admin;

import org.apache.log4j.Category;

import javax.servlet.*;
import javax.servlet.http.*;

import com.urbancode.anthill.*;

import java.net.URL;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.util.Properties;

/**
 *
 * @author  breese@grandcentral.com
 * @version
 */
public class RepositoryPropertiesViewServlet extends HttpServlet {
    
    //**************************************************************************
    // CLASS
    //**************************************************************************
    
    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(ProjectPropertiesViewServlet.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    /** Initializes the servlet.
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
    }
    
    /** Destroys the servlet.
     */
    public void destroy() {
        
    }
    
    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, java.io.IOException {
        log.debug("Processing request for RepositoryPropertiesViewServlet");
        ServletContext context = getServletContext();
        Anthill anthill = (Anthill)context.getAttribute(WebKeys.AnthillKey);
        AnthillProject project = null;
        String projectName = req.getParameter(WebKeys.ProjectNameKey);
        log.debug("projectName: " + projectName);
        boolean error = false;
        if (projectName != null) {
            project = anthill.getProject(projectName);
            String repositoryAdapter =
                        project.getProperties().getRepositoryAdapterName();
            log.debug("repositoryadapter: " + repositoryAdapter);
            try {
                Properties props = loadRepositoryConfigOptions(repositoryAdapter);
                if (project != null) {
                    req.setAttribute(WebKeys.AnthillKey, anthill);
                    req.setAttribute(WebKeys.ProjectKey, project);
                    req.setAttribute(WebKeys.RepositoryConfigKey, props);
                } else {
                    error = true;
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                error = true;
            }
        } else {
            error = true;
        }
        
        if (error) {
            context.getRequestDispatcher(ScreenNames.errorScreen).forward(req , res);
        } else {
            context.getRequestDispatcher(ScreenNames.repositoryPropScreen).forward(req , res);
        }
    }
    
    /** Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }
    
    /** Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }
    
    /** Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Short description";
    }
    
    private Properties loadRepositoryConfigOptions(String adapter)
    throws Exception {
        log.debug("Loading " + adapter);
//        Class repositoryAdapterClass = Class.forName(adapter);
//        ClassLoader cl = repositoryAdapterClass.getClassLoader();//project.getClass().getClassLoader();
        String adapterConfig = System.getProperty(Anthill.ANTHILL_ROOT_DIR_KEY) + 
                                File.separator + "conf" + File.separator + 
                                adapter + ".properties";
//        String adapterConfig = adapter.substring(adapter.lastIndexOf('.')+1) + ".properties";;
        log.debug("adapterConfig " + adapterConfig);
//        adapter.replace('.',File.separator.charAt(0)) + ".properties";
//        URL url = cl.getResource(adapterConfig);
//        if (url == null) {
//            String msg = "Could not find properties file: " + adapterConfig;
//            log.error(msg);
//            throw new IOException(msg);
//        }
//        InputStream rf = cl.getResourceAsStream(adapterConfig);
        InputStream rf = new FileInputStream(new File (adapterConfig));
        if (rf == null) {
            String msg = "Error creating inputstream for properties file: " + adapterConfig;
            log.error(msg);
            throw new IOException(msg);
        }
        Properties config_list = new Properties();
        config_list.load(rf);
        log.debug("Loaded properties file: " + adapterConfig);
        return config_list;
    }
    
}
