/*
 * @(#)ViewProjectServlet.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.web.admin;           
import org.apache.log4j.Category;
import javax.servlet.*;
import javax.servlet.http.*;
import com.urbancode.anthill.*;
import java.io.*;
/**
 * @version $Revision: 1.2 $
 * @author 
 */
public class ViewProjectServlet extends HttpServlet {
   
    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(ViewProjectServlet.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    /** Initializes the servlet.
    */  
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
    }

    /** Destroys the servlet.
    */  
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, java.io.IOException {
        ServletContext context = getServletContext();
        Anthill anthill = (Anthill)context.getAttribute(WebKeys.AnthillKey);
        
        // get the name of the local resource
        String rawResName = req.getPathInfo();
        log.debug("PathInfo: "+rawResName);
        
        // strip off the leading / since WebDirectory will not find the 
        // resource if we keep the leading /
        if (rawResName.startsWith("/")) {
            rawResName = rawResName.substring(1);
            log.debug("Resource Name: "+rawResName);
        }
        
        File projectsDir = anthill.getAnthillDefaultPublishDir();
        File resFile = new File(projectsDir, rawResName);
        log.debug("resFile: "+resFile);
        log.debug("resFile path: "+resFile.getAbsolutePath());
        
        if (resFile.exists()) {
            // the resource exists, good news
            
            if (resFile.isDirectory()) {
                // the resource is a directory
                // forward to the jsp that knows how to display directories
                req.setAttribute("dir", resFile);
                req.setAttribute("resName", rawResName);
                context.getRequestDispatcher("/directoryList.jsp").forward(req , res);
            } else {
                // the resource is a file
                // read in the file contents and write out to the output stream
                String contentType = context.getMimeType(rawResName);
                if (contentType != null) {
                    res.setContentType(contentType);
                }
                
                try {
                    OutputStream os = res.getOutputStream();
                    InputStream in = new BufferedInputStream(new FileInputStream(resFile));
                    StringBuffer requestBuf = new StringBuffer();
                    int numRead;
                    byte bytes[] = new byte[50];
                    while ((numRead = in.read(bytes)) >= 0) {
                        os.write(bytes, 0, numRead);
                    }
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                    throw new ServletException(e);
                }
                
            }
        } else {
            // the requested resource does not exist
            // forward to an error page
            context.getRequestDispatcher("/errorPage.jsp").forward(req , res);
        }
    } 

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Short description";
    }

}
