/*
 * PropertyFilter0to1.java
 *
 * Created on July 10, 2002, 11:15 AM
 */

package com.urbancode.anthill;
import com.urbancode.lib.registry.*;
import java.util.Map;
import java.util.Iterator;
import org.apache.log4j.Category;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Vector;
/**
 *
 * @author  spb
 */
public class PropertyFilter0to1 extends PropertyFilter {
    
    static private Category log = 
        Category.getInstance(PropertyFilter0to1.class.getName());
    
    /** Creates a new instance of PropertyFilter0to1 */
    public PropertyFilter0to1(Registry registry, File propertyFile) {
       
        projectRegEntry = registry.getRegistryEntry(REGISTRY_ENTRY_KEY); 
        String version = projectRegEntry.getKeyValue(REGISTRY_FORMAT_VERSION);
        log.debug("Version string: " + version);
        int upToDate = isUpToDate(version, "1");
        if (upToDate > 0) {
            // here they are at 1.1 or better
            // send them to the 1 to 1.1 filter and let it decide what to do
            PropertyFilter1to1pt1 pf = new PropertyFilter1to1pt1(registry, propertyFile);
            return; // don't run through this filter. 
        }
        else if (upToDate == 0) return; // don't need to run this filter
        
        // can't go backwards, so run this filter.  They must have been at 0
        updateAntParams(BUILD_SCRIPT_ANT_PARAMS);
        updateAntParams(PUBLISH_SCRIPT_ANT_PARAMS);
        convertSysToAnt(BUILD_SCRIPT_SYS_PARAMS);
        convertSysToAnt(PUBLISH_SCRIPT_SYS_PARAMS);
        projectRegEntry.addKeyValue(REGISTRY_FORMAT_VERSION, "1");
        
        // here you have to back the file up, and to do that you need the
        // path...
        backupFile(propertyFile, "0");
        
        try {
           registry.updateRegistryFile();
        } 
        catch (Exception e) {
           log.error(e.getMessage(), e);
        }
        // now call the next filter
        PropertyFilter1to1pt1 pf2 = new PropertyFilter1to1pt1(registry, propertyFile);
    }
    
    // so to get at the propertis, you first get yourself a registry entry
    // then from there you can retrieve the child entries using the PARAMS key
    
    // The sys params have to be converted to ant params, and they will be 
    // of the form -Dname=value.  The ant params need to be changed so that 
    // they are name value, not name=value
    
    // in the end, all you have are ant params, no sys params.  That brings
    // you up to version 1 from 0.  
    
    public void convertSysToAnt(String paramString) {
        RegistryEntry paramEntry = projectRegEntry.getChildRegistryEntry(paramString);
        if (paramEntry == null) {
            log.debug("No parameters to convert in convertSysToAnt");
            return;
        }
        Iterator regEntryItr = paramEntry.getChildRegistryEntries(PARAMS);
        Vector oldEntries = new Vector();
        if (regEntryItr != null) {
            while (regEntryItr.hasNext()) {
                RegistryEntry regEntry = (RegistryEntry)regEntryItr.next();
                String paramName = regEntry.getKeyValue("name");
                String paramValue = regEntry.getKeyValue("value");
                // so here we are already moving towards value only, 
                // but we do still store the name and value
                // the value is what is printed out to the screen
                // the name doesn't show up on the web page...
                // would be nice if we could update these, and not delete them..
                if (paramName != null && paramValue != null) {
                    if (paramString.equals(BUILD_SCRIPT_SYS_PARAMS)) { // build param
                        RegistryEntry newRegEntry = projectRegEntry.addKeyValue(BUILD_ANT_PARAM_NAME_KEY, 
                                                    "-D"+paramName+"="+paramValue);
                        newRegEntry.addKeyValue(VALUE, "-D"+paramName+"="+paramValue);
                    }
                    else {
                        RegistryEntry newRegEntry = projectRegEntry.addKeyValue(PUB_ANT_PARAM_NAME_KEY, 
                                                "-D"+paramName+"="+paramValue);
                        newRegEntry.addKeyValue(VALUE, "-D"+paramName+"="+paramValue);
                    }
                    oldEntries.addElement(regEntry);
                }
            }
            // here we are done iterating, you can safely delete things
            for (int i = 0; i < oldEntries.size(); i++) {
                RegistryEntry tempEntry = (RegistryEntry)oldEntries.elementAt(i);
                log.debug("Removing Name: " + tempEntry.getKeyValue("name") + 
                ", Value: " + tempEntry.getKeyValue("value"));
                paramEntry.removeChildRegistryEntry((RegistryEntry)oldEntries.elementAt(i));
            }
            oldEntries.removeAllElements();
            oldEntries = null;
        }
    }
    
    public void updateAntParams(String paramString) {
        // for this one, you go through the existing ant params, and you
        
        RegistryEntry paramEntry = projectRegEntry.getChildRegistryEntry(paramString);
        if (paramEntry == null) {
            log.debug("No parameters to update in updateAntParams");
            return; // no parameters - nothing to do
        }
        Iterator regEntryItr = paramEntry.getChildRegistryEntries(PARAMS);
        if (regEntryItr != null) {
            while (regEntryItr.hasNext()) {
                RegistryEntry regEntry = (RegistryEntry)regEntryItr.next();
                String paramName = regEntry.getKeyValue("name");
                String paramValue = regEntry.getKeyValue("value");
                if (paramName != null && paramValue != null) {
                    regEntry.setKeyValue("value", "-"+paramName+" "+paramValue);
                    // that updates the old value...I hope
                }
            }
        }
    }
}
