/*
 * PropertyFilter1to1pt1.java
 *
 * Created on July 10, 2002, 2:09 PM
 */

package com.urbancode.anthill;
import com.urbancode.lib.registry.*;
import java.util.Map;
import java.util.Iterator;
import org.apache.log4j.Category;
import java.io.File;
import java.io.FileWriter;
import java.io.FileReader;
import java.util.Vector;

/**
 *
 * @author  spb
 */
public class PropertyFilter1to1pt1 extends PropertyFilter {
    
    static private Category log = 
        Category.getInstance(PropertyFilter1to1pt1.class.getName());
    
    /** Creates a new instance of PropertyFilter1to1pt1 */
    public PropertyFilter1to1pt1(Registry registry, File propertyFile) {
       
        projectRegEntry = registry.getRegistryEntry(REGISTRY_ENTRY_KEY); 
        String version = projectRegEntry.getKeyValue(REGISTRY_FORMAT_VERSION);
        log.debug("Version string: " + version);
        int upToDate = isUpToDate(version, "1.1");
        if (upToDate > 0) {
            // here they are at 1.2 or better
            return; // don't run through this filter. 
        }
        else if (upToDate == 0) return; // don't need to run this filter
        // else we need to run the filter
                
        backupFile(propertyFile, "1");
        
        // do the update...
        removeNameEntries(BUILD_SCRIPT_ANT_PARAMS);
        removeNameEntries(PUBLISH_SCRIPT_ANT_PARAMS);
        
        projectRegEntry.setKeyValue(REGISTRY_FORMAT_VERSION, "1.1");
        
        try {
           registry.updateRegistryFile();
        } 
        catch (Exception e) {
           log.error(e.getMessage(), e);
        }
    }
    
    // the idea here is that we don't have to store the name and value, we 
    // can just store the value itself, more of a list now than a map
    public void removeNameEntries(String paramString) {
        RegistryEntry paramEntry = projectRegEntry.getChildRegistryEntry(paramString);
        if (paramEntry == null) {
            log.debug("Param entry for: " + paramString + " is null!");
            return;
        }
        Iterator regEntryItr = paramEntry.getChildRegistryEntries(PARAMS);
        Vector oldEntries = new Vector();
        Vector newEntries = new Vector();
        if (regEntryItr != null) {
            while (regEntryItr.hasNext()) {
                RegistryEntry regEntry = (RegistryEntry)regEntryItr.next();
                // don't need the name even, just the value, that's all we're taking
                String paramValue = regEntry.getKeyValue("value");
                oldEntries.addElement(regEntry);
                newEntries.addElement(paramValue);
            }
            // the iterator is now done, so we can safely add/delete 
            for (int i = 0; i < oldEntries.size(); i++) {
                paramEntry.removeChildRegistryEntry((RegistryEntry)oldEntries.elementAt(i));
            }
            for (int j = 0; j < newEntries.size(); j++) {
                if (paramString.equals(BUILD_SCRIPT_ANT_PARAMS)) {
                    projectRegEntry.addKeyValue(BUILD_ANT_PARAM_VALUE_KEY, 
                                                (String)newEntries.elementAt(j));
                }
                else {
                    projectRegEntry.addKeyValue(PUB_ANT_PARAM_VALUE_KEY, 
                                                (String)newEntries.elementAt(j));
                }
            }
            oldEntries.removeAllElements();
            newEntries.removeAllElements();
            oldEntries = newEntries = null;
        }
    }
}
